@@ -4,6 +4,7 @@ import json
 from aiohttp import ClientSession
 import asyncio
 import warnings
+import urllib
 
 
 class Tenant:
@@ -104,7 +105,9 @@ class Tenant:
         try:
             r = s.get(self.tenant + '/api/v1/' + endpoint, params=params)
             if r.status_code == 200:
-                result = r.json()['data']
+                result = r.json()
+                if 'data' in result:
+                    result = result['data']
                 try:
                     starting_after = re.findall(
                         next_re, r.json()['links']['next']['href'])[0]
@@ -117,6 +120,20 @@ class Tenant:
                     except IndexError:
                         params.clear()
                         return result
+                except KeyError:
+                    try:
+                        starting_after = re.findall(
+                            next_re, r.json()['links']['Next']['Href'])[0]
+                    except IndexError:
+                        try:
+                            next_re = r'(?<=&startingAfter=)(?:(?!&|$).)*'
+                            starting_after = re.findall(
+                                next_re, r.json()['links']['Next']['Href'])[0]
+                            next_name = 'startingAfter'
+                        except IndexError:
+                            params.clear()
+                            return result
+
             else:
                 params.clear()
                 raise Exception(r.status_code, r.text)
@@ -124,13 +141,17 @@ class Tenant:
                 params[next_name] = starting_after
                 r = s.get(self.tenant + '/api/v1/' + endpoint, params=params)
                 result += r.json()['data']
-                starting_after = re.findall(
-                    next_re, r.json()['links']['next']['href'])[0]
+                try:
+                    starting_after = re.findall(
+                        next_re, r.json()['links']['next']['href'])[0]
+                except KeyError:
+                    starting_after = re.findall(
+                        next_re, r.json()['links']['Next']['Href'])[0]
         except KeyError:
             pass
         except TypeError:
             params.clear()
-            return r.json()
+            return result
         s.close()
         params.clear()
         return result
@@ -156,7 +177,7 @@ class Tenant:
         s = requests.Session()
         s.headers.update(self.auth_header)
         r = s.delete(self.tenant + '/api/v1/' + endpoint)
-        if r.status_code >= 200 and r.status_code < 300:
+        if r.status_code in range(200, 300):
             try:
                 result = r.json()
             except json.decoder.JSONDecodeError:
@@ -298,7 +319,27 @@ class Tenant:
                 response = await resp.text()
                 if resp.status not in range(200, 300):
                     raise Exception(resp.status, response)
-                copied_app_id = json.loads(response)['attributes']['id']
+
+                attributes = json.loads(response)['attributes']
+                copied_app_id = attributes['id']
+                payload = {
+                    "name": attributes['name'],
+                    "resourceId": attributes['id'],
+                    "description": attributes['description'],
+                    "resourceType": "app",
+                    "resourceAttributes": attributes,
+                    "resourceCustomAttributes": {},
+                    "resourceCreatedAt": attributes['createdDate'],
+                    "resourceCreatedBySubject": attributes['owner']
+                }
+
+            async with session.post(url + 'items', data=json.dumps(payload),
+                                    headers=self.auth_header) as resp:
+                response = await resp.text()
+                if resp.status not in range(200, 300):
+                    raise Exception(resp.status, response)
+                return response
+
             if user_id:
                 async with session.put(url + 'apps/' + copied_app_id +
                                        '/owner',
@@ -574,6 +615,10 @@ class Tenant:
         Private helper function for _generic.
         """
         func = 's.' + method
+        if 'import' in endpoint:
+            params = urllib.parse.urlencode(
+                params, quote_via=urllib.parse.quote)
+
         if not json:
             r = eval(func)(self.tenant + '/api/v1/' + endpoint,
                            params=params, data=body)
@@ -589,7 +634,6 @@ class Tenant:
         --------------------
         Private helper function for post, put, patch.
         """
-
         flag_400 = False
         flag_500 = False
         s = requests.Session()
@@ -597,6 +641,11 @@ class Tenant:
 
         r = self._generic_request(s, method, endpoint, body, params)
 
+        if r.status_code == 415:
+            s.headers.update({'Content-Type': 'application/json',
+                              'Accept': 'application/json'})
+            r = self._generic_request(s, method, endpoint, body,
+                                      params)
         if r.status_code == 400:
             flag_400 = True
             body = [body]
@@ -620,7 +669,7 @@ class Tenant:
             r = self._generic_request(s, method, endpoint, body,
                                       params)
 
-        if r.status_code >= 200 and r.status_code < 300:
+        if r.status_code in range(200, 300):
             try:
                 result = r.json()
             except json.decoder.JSONDecodeError:
